#! /bin/sh

#
# Change into the directory containing this script, so that we know
# the correct paths.
#
cd `echo $0 | sed -e 's,[^/]*$,,;s,/$,,;s,^$,.,'`

#
# Determine the current version number.
#
VERS=`sed -n -e '/^#define[	 ][	 ]*VERS/p' ../src/petidomo/version.h | sed -e 's/^.*"\(.*\)"/\1/' -e 's/ /-/g' -e 's/-beta-/b/g'`

#
# Determine the id-string of the platform.
#
MACHINE=`(uname -mrs) 2>/dev/null` || MACHINE=unknown
MACHINE=`echo $MACHINE | tr [A-Z] [a-z] | sed -e 's/ /-/g'`

#
# Copy all required files into a temporary directory.
#
TMPDIR=/tmp/petidomo-$MACHINE
rm -rf $TMPDIR
./bsdinstall -d $TMPDIR
./bsdinstall -d $TMPDIR/bin
./bsdinstall -d $TMPDIR/etc
./bsdinstall -d $TMPDIR/lists
./bsdinstall -d $TMPDIR/crash
./bsdinstall -c -s ../src/petidomo/petidomo $TMPDIR/bin/petidomo
ln $TMPDIR/bin/petidomo $TMPDIR/bin/hermes
ln $TMPDIR/bin/petidomo $TMPDIR/bin/listserv
./bsdinstall -c ./send-pr $TMPDIR/bin/send-pr
./bsdinstall -c ./rfc2369.sh $TMPDIR/bin/rfc2369.sh
./bsdinstall -c ./InsertNameInSubject.sh $TMPDIR/bin/InsertNameInSubject.sh
./bsdinstall -c ./pgp-encrypt.sh $TMPDIR/bin/pgp-encrypt.sh
./bsdinstall -c ./pgp-decrypt.sh $TMPDIR/bin/pgp-decrypt.sh
./bsdinstall -c -s ../src/htmlconf/htmlconf $TMPDIR/bin/petidomoconf.cgi
./bsdinstall -c ../etc/help $TMPDIR/etc
./bsdinstall -c ../etc/masteracl $TMPDIR/etc/acl
./bsdinstall -c ../etc/masterconfig $TMPDIR/etc/petidomo.conf

#
# Create the inner tar file.
#
(
    cd $TMPDIR
    tar -cf petidomo-bin.tar bin crash etc lists
    chmod 600 petidomo-bin.tar
    rm -rf bin crash etc lists
)


#
# Copy the README, COPYRIGHT and the install.sh script into the
# directory.
#
./bsdinstall -c -m 644 ../etc/README $TMPDIR/README
./bsdinstall -c -m 644 ../etc/COPYRIGHT $TMPDIR/COPYRIGHT
./bsdinstall -c -m 755 ./install-from-bindist.sh $TMPDIR/install.sh

#
# Create the release archive.
#
(
    echo Creating $VERS-$MACHINE.tar.gz
    cd /tmp
    tar -cf $VERS-$MACHINE.tar petidomo-$MACHINE
    gzip $VERS-$MACHINE.tar
)

#
# Clean up
#
rm -rf $TMPDIR
