/*
 *      $Source: /usr/local/libdata/cvs/simons/petidomo/src/htmlconf/handlecreatlist.c,v $
 *      $Revision: 1.12 $
 *      $Date: 1998/08/17 16:31:00 $
 *
 *      Copyright (C) 1996 by Peter Simons, Germany
 *      All rights reserved.
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <text.h>
#include <cgi.h>
#include <petidomo.h>

extern cgi_info      ci;
extern form_entry *  form_parameter;

void printMainPage(void);
int  createMailinglistAlias(const char *);

const char listacl[] = \
"#\n" \
"# Generic list acl file -- written by htmlconf\n" \
"#\n" \
"\n" \
"# Re-directing anything that looks like a command.\n" \
"IF   (subject matches \"^[	 ]*((un)?subscribe|remove|delete)\")\n" \
"  OR (body matches \"^[	 ]*((un)?subscribe|remove|delete)( me)?$\")\n" \
"	then redirect \"%s-request\";\n" \
;

static void
strip_lf(char * buffer)
{
    char *   in  = buffer,
	 *   out = buffer;

    while(*in != '\0') {
	if (*in == '\r') {
	    if (*(in+1) == '\r' || *(in+1) == '\n') {
		in++;
	    }
	    else {
		in++;
		*out++ = '\n';
	    }
	}
	else
	  *out++ = *in++;
    }
    *out++ = '\0';
}

void
handleBlanks(char ** option)
{
    char *   new_option;

    if (strchr(*option, '\"') != NULL)
      return;			/* already contains brackets */

    if (strchr(*option, ' ') == NULL)
      return;			/* no blanks */

    new_option = xmalloc(strlen(*option)+3);
    *new_option = '\"';
    strcpy(new_option+1, *option);
    strcat(new_option, "\"");
    *option = new_option;
}

void
handleCreateListPage(void)
{
    const struct PD_Config * MasterConfig = getMasterConfig();
    FILE *     fh;
    char *     filename;
    char *     password = parmval(form_parameter, "password");
    char *     listname = parmval(form_parameter, "listname");
    char *     description = parmval(form_parameter, "description");
    char *     introduction = parmval(form_parameter, "introduction");
    char *     signature = parmval(form_parameter, "signature");
    char *     Hostname = parmval(form_parameter, "Hostname");
    char *     adminpassword = parmval(form_parameter, "adminpassword");
    char *     adminpassword2 = parmval(form_parameter, "adminpassword2");
    char *     allowaliensub = parmval(form_parameter, "allowaliensub");
    char *     allowmembers = parmval(form_parameter, "allowmembers");
    char *     allowpubsub = parmval(form_parameter, "allowpubsub");
    char *     archivepath = parmval(form_parameter, "archivepath");
    char *     listtype = parmval(form_parameter, "listtype");
    char *     postingfilter = parmval(form_parameter, "postingfilter");
    char *     postingpassword = parmval(form_parameter, "postingpassword");
    char *     postingpassword2 = parmval(form_parameter, "postingpassword2");
    char *     replyto = parmval(form_parameter, "replyto");
    char *     showonindex = parmval(form_parameter, "showonindex");
    int        rc;

    strip_lf(introduction);
    strip_lf(description);
    strip_lf(signature);

    /* Do consistency checks. */

    if (isValidAdminPassword(password, NULL) == FALSE) {
	puts("<H1 ALIGN=center>Access denied!</H1>");
	puts("The password you entered does not authorize you to create the");
	puts("mailing list, I am afraid.");
	return;
    }

    if (listname == NULL) {
	puts("<H3>ERROR</H1>");
	puts("You didn't specify the name of the list to be created. Please use the \n");
	puts("\"previous page\" function of your browser to edit the \n");
	puts("entries in the form again.<P>");
	return;
    }

    if (strchr(listname, '/') != NULL) {
	puts("<H3>ERROR</H1>");
	puts("The mailing list name must not contain a '/'. Please use the \n");
	puts("\"previous page\" function of your browser to edit the \n");
	puts("entries in the form again.<P>");
	return;
    }

    if (strcasecmp(adminpassword, adminpassword2) != 0) {
	puts("<H3>ERROR</H1>");
	puts("You mis-typed the administrator password. Please use the \n");
	puts("\"previous page\" function of your browser to edit the \n");
	puts("entries in the form again.<P>");
	return;
    }

    if (strcasecmp(postingpassword, postingpassword2) != 0) {
	puts("<H3>ERROR</H1>");
	puts("You mis-typed the posting password. Please use the \n");
	puts("\"previous page\" function of your browser to edit the \n");
	puts("entries in the form again.<P>");
	return;
    }

    handleBlanks(&adminpassword);
    handleBlanks(&postingpassword);
    handleBlanks(&postingfilter);

    /* Create the mailing list directory. */

    filename = text_easy_sprintf("lists/%s", listname);
    rc = mkdir(filename, 0750);
    if (rc != 0) {
	puts("<H3>ERROR</H1>");
	printf("Creating the directory \"~petidomo/%s\" failed with the following error:<P>\n",
	       filename);
	printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	puts("Please contact the administrator with this problem.<P>\n");
	return;
    }

    /* Write the description. */

    if (description != NULL && *description != '\0') {
	filename = text_easy_sprintf("lists/%s/description", listname);
	fh = fopen(filename, "w");
	if (fh == NULL) {
	    puts("<H3>ERROR</H1>");
	    printf("Opening the 'description' file \"%s\" failed with the following error:<P>\n",
		   filename);
	    printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	    puts("Please contact the administrator with this problem.<P>\n");
	    return;
	}
	fprintf(fh, "%s", description);
	if (description[strlen(description)-1] != '\n')
	  fprintf(fh, "\n");
	fclose(fh);
    }

    /* Write the introduction. */

    if (introduction != NULL && *introduction != '\0') {
	filename = text_easy_sprintf("lists/%s/introduction", listname);
	fh = fopen(filename, "w");
	if (fh == NULL) {
	    puts("<H3>ERROR</H1>");
	    printf("Opening the 'introduction' file \"%s\" failed with the following error:<P>\n",
		   filename);
	    printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	    puts("Please contact the administrator with this problem.<P>\n");
	    return;
	}
	fprintf(fh, "%s", introduction);
	if (introduction[strlen(introduction)-1] != '\n')
	  fprintf(fh, "\n");
	fclose(fh);
    }

    /* Write the signature. */

    if (signature != NULL && *signature != '\0') {
	filename = text_easy_sprintf("lists/%s/signature", listname);
	fh = fopen(filename, "w");
	if (fh == NULL) {
	    puts("<H3>ERROR</H1>");
	    printf("Opening the 'signature' file \"%s\" failed with the following error:<P>\n",
		   filename);
	    printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	    puts("Please contact the administrator with this problem.<P>\n");
	    return;
	}
	fprintf(fh, "%s", signature);
	if (signature[strlen(signature)-1] != '\n')
	  fprintf(fh, "\n");
	fclose(fh);
    }

    /* Write the acl file. */

    filename = text_easy_sprintf("lists/%s/acl", listname);
    fh = fopen(filename, "w");
    if (fh == NULL) {
	puts("<H3>ERROR</H1>");
	printf("Opening the 'acl' file \"%s\" failed with the following error:<P>\n",
	       filename);
	printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	puts("Please contact the administrator with this problem.<P>\n");
	return;
    }
    fprintf(fh, listacl, listname);
    fclose(fh);

    /* Write a dummy list file. */

    filename = text_easy_sprintf("lists/%s/list", listname);
    fh = fopen(filename, "w");
    if (fh != NULL)
      fclose(fh);
    else {
	puts("<H3>ERROR</H1>");
	printf("Opening the list file \"%s\" failed with the following error:<P>\n",
	       filename);
	printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	puts("Please contact the administrator with this problem.<P>\n");
	return;
    }

    /* Write the new config file back. */

    filename = text_easy_sprintf("lists/%s/config", listname);
    fh = fopen(filename, "w");
    if (fh == NULL) {
	puts("<H3>ERROR</H1>");
	printf("Opening the config file \"%s\" failed with the following error:<P>\n",
	       filename);
	printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	puts("Please contact the administrator with this problem.<P>\n");
	return;
    }
    fprintf(fh, "#\n");
    fprintf(fh, "# Config file for mailing list \"%s\"\n", listname);
    fprintf(fh, "# generated automatically by petidomoconf\n");
    fprintf(fh, "#\n");
    fprintf(fh, "ListType\t\t%s\n", listtype);
    fprintf(fh, "AllowPublicSubscription\t%s\n", allowpubsub);
    fprintf(fh, "AllowAlienSubscription\t%s\n", allowaliensub);
    fprintf(fh, "AllowMembersCommand\t%s\n", allowmembers);
    fprintf(fh, "ShowOnIndex\t\t%s\n", showonindex);
    if (!strcasecmp(MasterConfig->fqdn, Hostname))
      fprintf(fh, "#");
    fprintf(fh, "Hostname\t\t%s\n", Hostname);
    if (*adminpassword == '\0')
      fprintf(fh, "#AdminPasswort\t\t\"open sesame\"\n");
    else
      fprintf(fh, "AdminPassword\t\t%s\n", adminpassword);
    if (*postingpassword == '\0')
      fprintf(fh, "#PostingPasswort\t\"lemme post\"\n");
    else
      fprintf(fh, "PostingPassword\t\t%s\n", postingpassword);
    if (*replyto == '\0')
      fprintf(fh, "#ReplyTo\t\tnone\n");
    else
      fprintf(fh, "ReplyTo\t\t%s\n", replyto);
    if (*postingfilter == '\0')
      fprintf(fh, "#PostingFilter\t\t\"/bin/cat\"\n");
    else
      fprintf(fh, "PostingFilter\t\t%s\n", postingfilter);
    if (*archivepath == '\0')
      fprintf(fh, "#Archive\t\t\"log\"\n");
    else
      fprintf(fh, "Archive\t\t%s\n", archivepath);
    fclose(fh);

    /* Create the aliases for the list. */

    rc = createMailinglistAlias(listname);
    if (rc != 0) {
	printMainPage();
	puts("<H3>ERROR</H1>");
	puts("The mailing list was created, but adding the required aliases to\n");
	puts("\"/etc/aliases\" failed with the following error:<P>\n");
	printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	puts("The mailing list aliases have to be added manually.\n");
	puts("Please contact the administrator with this problem.<P>\n");
	return;
    }

    /* If all was successful, show the main page again. */

    printMainPage();
    puts("<H3>Success</H1>");
    puts("The new mailing list has been created successfully.<P>");
}
