/*
 *      $Source: /usr/local/libdata/cvs/simons/petidomo/src/htmlconf/printremlist.c,v $
 *      $Revision: 1.5 $
 *      $Date: 1998/06/01 17:19:29 $
 *
 *      Copyright (C) 1997 by Peter Simons, Germany
 *      All rights reserved.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <cgi.h>
#include <petidomo.h>

extern cgi_info      ci;
extern form_entry *  form_parameter;

void printMainPage(void);
bool isValidListName(const char *);

const char remove_html_pre[] = \
"<H1 ALIGN=CENTER>Petidomo 2.2 Mailinglist Removal</H1>\n" \
"<FORM METHOD=\"POST\" ACTION=\"%s\">\n" \
"<INPUT TYPE=hidden NAME=page VALUE=remlist>\n" \
"<INPUT TYPE=hidden NAME=password VALUE=\"%s\">\n" \
"Please select the mailing list, you'd like to remove: <BR>\n" \
"<CENTER>\n" \
"<TABLE>\n";

const char remove_html_mid[] = \
"</TABLE>\n" \
"</CENTER><P>\n" \
"<INPUT TYPE=\"submit\"><INPUT TYPE=\"reset\"><P>\n";

const char remove_html_post[] = \
"</FORM>\n" \
"</BODY>\n" \
"</HTML>\n";

void
printRemList(void)
{
    char *           password = parmval(form_parameter, "password");
    DIR *            dirp;
    struct dirent *  entry;
    unsigned int     entry_num;

    /* Do sanity checks. */

    if (password == NULL) {
	printMainPage();
	return;
    }

    /* Check the password. */

    if (isValidAdminPassword(password, NULL) == FALSE) {
        printMainPage();
        puts("<H3>Permission denied</H1>");
        puts("The password you entered does not authorize you to remove\n");
        puts("a mailing list.<P>\n");
        return;
    }

    /* Print Preamble. */

    printf(remove_html_pre, ci.script_name, password);

    /* Generate a list of available mailing list. */

    entry_num = 0;
    dirp = opendir("lists");
    if (dirp == NULL) {
	printf("An internal error has occured while processing your request. The\n" \
	       "server administrator has been notified. You don't need to re-submit\n" \
	       "your request, it will be processed as soon as the problem has been\n" \
	       "remedied.\n");
	syslog(LOG_ERR, "Failed to read directory \"lists\": %m");
	exit(0);
    }
    while((entry = readdir(dirp)) != NULL) {
	if (!strcasecmp(entry->d_name, ".") || !strcasecmp(entry->d_name, ".."))
	  continue;
	if (isValidListName(entry->d_name) == FALSE)
	  continue;
	entry_num++;

	/* Print stuff to the page. */

	printf("  <TR><TD><INPUT TYPE=radio NAME=listname VALUE=\"%s\"> %s </TR>\n",
	       entry->d_name, entry->d_name);
    }
    closedir(dirp);

    /* Print Midfix. */

    puts(remove_html_mid);

    if (entry_num == 0) {
	puts("No mailing lists found.\n");
    }
    else
      printf("Found %d mailing lists.\n", entry_num);

    /* Print postfix. */

    puts("<P>\n");
    puts("<CENTER><STRONG>ATTENTION:<BR>The list is deleted immediately, no more\n");
    puts("'are you sure'-requesters! If you hit that 'Submit'-button, the list\n");
    puts("and all files belonging to it are gone.\n</STRONG></CENTER><P>\n");
    puts(remove_html_post);
}
